library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity freq_meas is
	Port (		
	    -- PWM input (only i_pwm(0) is used)
		i_pwm : in std_logic_vector(2 downto 0);
		
		-- Mean frequency of the PWM in kHz in a window of 1 ms (AXIS interface)
		o_freq_khz_tdata : out std_logic_vector(31 downto 0) := (others => '0');
		o_freq_khz_tvalid : out std_logic := '0';
		i_freq_khz_tready : in std_logic;
		
		-- Pulse indicating an update of the o_freq_khz signal
		o_update_pulse : out std_logic := '0';
		
		-- 250MHz clock
		clk_250_mhz : in std_logic
	);
end freq_meas;

architecture Behavioral of freq_meas is

	signal r_time_counter	       : std_logic_vector(18-1 downto 0) := (others => '0');
	signal r_rising_edge_counter   : std_logic_vector(31 downto 0) := (others => '0');
	signal r_pwm				   : std_logic := '0';
	signal r_freq_khz_tvalid_cp    : std_logic := '0';

begin

    P: process(clk_250_mhz)
    begin
        if rising_edge(clk_250_mhz) then
			r_pwm <= i_pwm(0);
			o_update_pulse <= '0';
			
			-- Time counter update
			if unsigned(r_time_counter) >= to_unsigned(250000, r_time_counter'length) then
				r_time_counter <= (others => '0');
				o_freq_khz_tdata <= r_rising_edge_counter;
				r_freq_khz_tvalid_cp <= '1';
				o_freq_khz_tvalid <= '1';
				o_update_pulse <= '1';
				r_rising_edge_counter <= (others => '0');
			else
				r_time_counter <= std_logic_vector(unsigned(r_time_counter) + 1);
				-- Rising edge counter update
				if (r_pwm = '0') and (i_pwm(0) = '1') then
					r_rising_edge_counter <= std_logic_vector(unsigned(r_rising_edge_counter) + 1);
				end if;
			end if;
			
			if (r_freq_khz_tvalid_cp = '1') and (i_freq_khz_tready = '1') then
            	r_freq_khz_tvalid_cp <= '0';
            	o_freq_khz_tvalid <= '0';
			end if;
		end if;
    end process P;

end Behavioral;
