library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity sw_freq_meas is
	Port (		
	    -- PWM input (only i_pwm(0) is used)
		i_pwm : in std_logic_vector(2 downto 0);
		
		-- Mean frequency of the PWM in kHz in a window of 1 ms (AXIS interface)
		M_AXIS_FREQU_tdata : out std_logic_vector(31 downto 0) := (others => '0');
		M_AXIS_FREQU_tvalid : out std_logic := '0';
		M_AXIS_FREQU_tready : in std_logic;
				
		-- 250MHz clock
		clk_250_mhz : in std_logic
	);
end sw_freq_meas;

architecture Behavioral of sw_freq_meas is

    ATTRIBUTE X_INTERFACE_INFO : STRING;
	ATTRIBUTE X_INTERFACE_PARAMETER : STRING;
	ATTRIBUTE X_INTERFACE_INFO of clk_250_mhz: SIGNAL is "xilinx.com:signal:clock:1.0 clk_250_mhz CLK";
	
	ATTRIBUTE X_INTERFACE_INFO of M_AXIS_FREQU_tdata:  SIGNAL is "xilinx.com:interface:axis_rtl:1.0 M_AXIS_FREQU tdata";
    ATTRIBUTE X_INTERFACE_INFO of M_AXIS_FREQU_tvalid: SIGNAL is "xilinx.com:interface:axis_rtl:1.0 M_AXIS_FREQU tvalid";	
	ATTRIBUTE X_INTERFACE_INFO of M_AXIS_FREQU_tready: SIGNAL is "xilinx.com:interface:axis_rtl:1.0 M_AXIS_FREQU tready";
	
	-- Here to avoid "missing ASSOCIATED_BUSIF" warning
	-- This parameter is mainly useful for AXI interface, when letting Vivado handling the interconnect
	ATTRIBUTE X_INTERFACE_PARAMETER of clk_250_mhz: SIGNAL is "ASSOCIATED_BUSIF M_AXIS_FREQU";

	signal r_time_counter	       : std_logic_vector(18-1 downto 0) := (others => '0');
	signal r_rising_edge_counter   : std_logic_vector(31 downto 0) := (others => '0');
	signal r_pwm				   : std_logic := '0';
	signal r_m_axis_freq_tvalid_cp : std_logic := '0';

begin

    P: process(clk_250_mhz)
    begin
        if rising_edge(clk_250_mhz) then
			r_pwm <= i_pwm(0);
			
			-- Time counter update
			if unsigned(r_time_counter) >= to_unsigned(250000, r_time_counter'length) then
				r_time_counter <= (others => '0');
				M_AXIS_FREQU_tdata <= r_rising_edge_counter;
				r_m_axis_freq_tvalid_cp <= '1';
				M_AXIS_FREQU_tvalid <= '1';
				r_rising_edge_counter <= (others => '0');
			else
				r_time_counter <= std_logic_vector(unsigned(r_time_counter) + 1);
				
				-- Rising edge counter update
				if (r_pwm = '0') and (i_pwm(0) = '1') then
					r_rising_edge_counter <= std_logic_vector(unsigned(r_rising_edge_counter) + 1);
				end if;
			end if;
			
			-- Valid signal reset
			if (r_m_axis_freq_tvalid_cp = '1') and (M_AXIS_FREQU_tready = '1') then
            	r_m_axis_freq_tvalid_cp <= '0';
            	M_AXIS_FREQU_tvalid <= '0';
			end if;
		end if;
    end process P;

end Behavioral;
